\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{exam}

%% Document Baseclass and encoding
\LoadClass[11pt, a4paper]{scrartcl}
\RequirePackage[T1]{fontenc}
\RequirePackage[utf8]{inputenc}
\RequirePackage{lmodern}
%%

%% Package Options
\newif\ifanswers\answersfalse
\newif\ifenglish\englishfalse

\DeclareOption{answers}{\answerstrue}
\DeclareOption{english}{\englishtrue}

\ProcessOptions
%%

%% Language Formatting
\ifenglish
\RequirePackage[english]{babel}
\else
\RequirePackage[ngerman]{babel}
\fi
%%

% General Layout
\RequirePackage[left=2cm, top=3cm]{geometry}
\setlength{\parindent}{0pt}
\setlength{\textwidth}{17cm}
\setlength{\textheight}{23.5cm}

\RequirePackage{scrlayer-scrpage}
\pagestyle{scrheadings}

\setlength\headheight{35pt}
\chead[]{
	\renewcommand{\arraystretch}{2}%
	\smash{%
		\ifthenelse{\isodd{\thepage}}{%
			\begin{tabularx}{\textwidth}{|p{9cm}|X|}\hline%
			\emph{Name:} & \emph{\ifenglish Matriculation Number \else Matrikelnummer\fi:} \\ \hline%
			\end{tabularx}%
		}{}%
	}%
	\renewcommand{\arraystretch}{1}%
}
\rohead[]{}
\rehead[]{}
\cfoot[]{--~\pagemark~--}
%%

\RequirePackage{amsmath, amssymb, amsthm,latexsym, graphicx}

\newtheoremstyle{examthmstyle}
    {\topsep}{\topsep}{\itshape}{}{\bfseries}{}{.5em}
    {\thmname{#1}\thmnumber{ #2}\thmnote{ \textmd{#3}}}
\theoremstyle{examthmstyle}

\RequirePackage{ifthen}
\RequirePackage{xspace}
\RequirePackage{xparse}
\RequirePackage{framed}
\RequirePackage{tabularx}
\RequirePackage{tikz}
\RequirePackage{tikz-qtree}
\usetikzlibrary{arrows,automata, petri, backgrounds,shapes, trees,positioning}

\makeatletter

\newcommand{\stickerbox}{\begin{center}\framebox{\begin{minipage}[c][50.8mm][c]{96mm}\centering -- \ifenglish Exam Supervisor's Sticker \else Aufkleber der Klausuraufsicht\fi --\end{minipage}}\end{center}}

\newcommand{\gradebox}{\begin{center}
	{\large \ifenglish Exam Grade \else Klausurnote\fi:} \fbox{\rule[-0.4cm]{0cm}{1cm}\rule{2cm}{0cm}}
	{\large ~Bonus:} \fbox{\rule[-0.4cm]{0cm}{1cm}\rule{2cm}{0cm}}
	{\large ~\ifenglish Final Grade \else Endnote\fi:} \fbox{\rule[-0.4cm]{0cm}{1cm}\rule{2cm}{0cm}}
	\end{center}
}

\ExplSyntaxOn
\NewDocumentCommand{\newenvcommand}{ m m } % #1 = env name, #2 = command name
  {
   \cs_if_exist:cF { g_envc_#1_list_tl } { \tl_new:c { g_envc_#1_list_tl } }
   \tl_gput_right:cn { g_envc_#1_list_tl } { #2 }
   \exp_after:wN \newcommand \cs:w envc_#1_\cs_to_str:N #2 \cs_end:
  }
\NewDocumentCommand{\checkenvcommands}{ }
  {
   \cs_if_exist:cT { g_envc_\use:c {@currenvir} _list_tl }
     {
      \tl_map_inline:cn { g_envc_\use:c {@currenvir} _list_tl }
        { \cs_set_eq:Nc ##1 { envc_\use:c {@currenvir} _\cs_to_str:N ##1 } }
     }
  }
\ExplSyntaxOff

\newenvironment{exercise}[2]{
	\global\advance\exam@numexercises by \@ne
	\global\advance\exam@totalpoints by #2\relax

	\global\edef\rows@exercise{\rows@exercise \string\node \space at (\the\exam@numexercises.5,1.25) {\the\exam@numexercises};}
	\global\edef\rows@maxpoints{\rows@maxpoints \string\node \space at (\the\exam@numexercises.5,.75) {#2};}
	\newpage
	\begin{exam@exercise}[(#1)\hfill\hspace*{15pt}/~#2 Punkte]\end{exam@exercise}
}{}
\newtheorem{exam@exercise}{\ifenglish Exercise \else Aufgabe\fi}

\def\RestOfPage{\dimexpr\pagegoal-\pagetotal-\baselineskip\relax}
\ifanswers
	\newenvironment{answer}[1][]
		{\par\begin{framed}\begin{minipage}{0.9\columnwidth}}
		{\end{minipage}\end{framed}}
\else
	\newsavebox{\trashcan}
	\newenvironment{answer}[1][]
		{
			\ifx\relax#1\relax
				\def\HeightOfBox{\RestOfPage}
			\else
				\def\HeightOfBox{#1}
			\fi
			\ifthenelse{\equal{\HeightOfBox}{nobox}}
				{}
				{\begin{framed}}
			\begin{lrbox}{\trashcan}\begin{minipage}{0.9\columnwidth}\hfill
		}
		{\end{minipage}\end{lrbox}
			\ifthenelse{\equal{\HeightOfBox}{nobox}}
				{}
				{\ifthenelse{\equal{\HeightOfBox}{}}
					{\phantom{\usebox{\trashcan}}\par\phantom{\usebox{\trashcan}}}
					{\vspace{\HeightOfBox}}
					\end{framed}
				}
		}
\fi

\newenvironment{exerciseblock}[1]{
	\global\advance\exam@numexercises by \@ne
	\newcount\pointsinsubexercises
	\newcount\numsubexercises
	\newpage
	\begin{exam@exercise}[(#1)]\end{exam@exercise}
}{
	\global\advance\exam@totalpoints by \pointsinsubexercises

	\global\edef\rows@exercise{\rows@exercise \string\node \space at (\the\exam@numexercises.5,1.25) {\the\exam@numexercises};}
	\global\edef\rows@maxpoints{\rows@maxpoints \string\node \space at (\the\exam@numexercises.5,.75) {\the\pointsinsubexercises};}
	\global\edef\rows@corrtable{\rows@corrtable \string\foreach \string\y \space in {1,...,\the\numsubexercises} \string\draw[dotted] (\the\exam@numexercises, .5*\string\y+1) -- (\the\exam@numexercises, .5*\string\y+1.5) -- (\the\exam@numexercises+1,.5*\string\y+1.5) -- (\the\exam@numexercises+1,.5*\string\y+1);}
}
\newenvironment{subexercise}[2]{
	\global\advance\pointsinsubexercises by #2\relax
	\global\advance\numsubexercises by \@ne
	\begin{exam@subexercise}[(#1)\hfill\hspace*{15pt}/~#2 Punkte]\end{exam@subexercise}
}{}
\newtheorem{exam@subexercise}{\ifenglish Subexercise \else Teilaufgabe \fi}[exam@exercise]

\newcommand\savefill[1]{{%
      \unskip\nobreak\hfil\penalty50
      \hskip2em\hbox{}\nobreak\hfil#1%
      \parfillskip=0pt \finalhyphendemerits=0 \par}}
\newcommand{\points}[1]{\savefill{\hspace*{10pt}/~#1~\ifnum#1=1 \ifenglish Point\else Punkt\fi \else \ifenglish Points\else Punkte\fi\fi}}

\newcommand{\ans}[1]{\ifanswers #1 \else \fi}

\newenvcommand{mcexercise}{\correct}[1]{\item #1\par\ifanswers $\boxtimes$ \ifenglish True \else Richtig \fi\hspace*{40pt}$\square$ \ifenglish False \else Falsch \fi\bigskip\else$\square$ \ifenglish True \else Richtig \fi\hspace*{40pt}$\square$ \ifenglish False \else Falsch \fi\bigskip\fi}
\newenvcommand{mcexercise}{\wrong}[1]{\item #1\par\ifanswers $\square$ \ifenglish True \else Richtig \fi\hspace*{40pt}$\boxtimes$ \ifenglish False \else Falsch \fi\bigskip\else$\square$ \ifenglish True \else Richtig \fi\hspace*{40pt}$\square$ \ifenglish False \else Falsch \fi\bigskip\fi}
\newenvironment{mcexercise}[3]
	{\checkenvcommands\begin{exercise}{#1}{#2}#3\begin{enumerate}}
	{\end{enumerate}\end{exercise}}

\usepackage{latexsym}
\newcommand{\bsCheckBoxN}{\raisebox{-1pt}{\parbox{1.5em}{\huge $\Box$}}}
\newcommand{\bsCheckBoxY}{\raisebox{-1pt}{\parbox{1.5em}{{\huge $\Box$}\raisebox{3pt}{\large\kern-1em X}}}}
\newcommand{\choice}[1][]{
  \item[\ifanswers{\ifx\\#1\\\bsCheckBoxN\else\bsCheckBoxY\fi}\else\bsCheckBoxN\fi]}
\newcommand{\newchoice}[1][]{
  \ifanswers{\ifx\\#1\\\bsCheckBoxN\else\bsCheckBoxY\fi}\else\bsCheckBoxN\fi}
\newenvironment{multiplechoice}
  {\begin{itemize}\setlength{\itemsep}{0ex}\vspace{-1ex}}
  {\end{itemize}}

\newcount\exam@numexercises
\newcount\exam@numexercises@pre
\newcount\exam@totalpoints
\newcount\exam@totalpoints@pre
\newcount\exam@numpages
\newcount\exam@numpages@pre
\def\rows@exercise{}
\def\rows@maxpoints{}
\def\rows@corrtable{}

\newcommand{\pointstable}{
	\begin{center}
	\IfFileExists{\jobname.ptb}{
	\begin{tikzpicture}[scale=1.1]
		\pgfmathsetmacro\tablewidth{\the\exam@numexercises@pre+1}
		\foreach \y in {0,...,2}{
			\draw (1, .5*\y) -- (-2.25, .5*\y) -- (-2.25, .5*\y+.5) -- (1, .5*\y+.5) -- (1, .5*\y);
			\foreach \x in {1,...,\tablewidth}
				\draw (\x, .5*\y) -- (\x, .5*\y+.5) -- (\x+1, .5*\y+.5) -- (\x+1, .5*\y) -- (\x, .5*\y);
		}
		\draw[ultra thick] (\tablewidth, 1.5) -- (\tablewidth, 0);

		\node[anchor = north west] at (-2.25, 1.5) {\ifenglish Exercise \else Aufgabe\fi};
		\node[anchor = north west] at (-2.25, 1) {\ifenglish maximum Points \else maximale Punkte\fi};
		\node[anchor = north west] at (-2.25, .5) {\ifenglish graded Points \else erreichte Punkte\fi};
		\node at (\tablewidth+.5, 1.25) {$\sum$};
		\input{\jobname.ptb}
	\end{tikzpicture}
	}{}
	\end{center}
}

\AtBeginDocument{
 \exam@numexercises@pre=\exam@numexercises\relax
 \exam@numexercises=0
 \exam@totalpoints@pre=\exam@totalpoints\relax
 \newcount\exam@passingpoints
 \advance\exam@passingpoints by \the\exam@totalpoints@pre
 \divide\exam@passingpoints by 2


 \exam@numpages@pre=\exam@numpages\relax

 \RequirePackage{lastpage}
 \newcommand{\maxpoints}{\the\exam@totalpoints@pre \xspace}
 \newcommand{\passingpoints}{\the\exam@passingpoints \xspace}
 \newcommand{\numexercises}{\the\exam@numexercises@pre \xspace}
 \newcommand{\numpages}{\the\exam@numpages@pre \xspace}
 \exam@totalpoints=0
}

%% Makro für Dokumentende
\AtEndDocument{
	~
  \begingroup\makeatletter
  	\addtocontents{pointdict}{\rows@exercise}
	\addtocontents{pointdict}{\rows@maxpoints \string\node \space at (\the\exam@numexercises+1.5, .75) {\the\exam@totalpoints};}
	\addtocontents{pointdict}{\rows@corrtable}
  \if@filesw
    \expandafter\newwrite\csname tf@pointdict\endcsname
    \immediate\openout\csname tf@pointdict\endcsname\jobname.ptb\relax
		\immediate\write\@auxout{\string\global\string\exam@totalpoints=\the\exam@totalpoints}
		\immediate\write\@auxout{\string\global\string\exam@numexercises=\the\exam@numexercises}
		\immediate\write\@auxout{\string\global\string\exam@numpages=\thepage}
  \fi
  \@nobreakfalse \endgroup

}

\makeatother

\newcommand{\hideme}[1]{ }

\newcommand{\N}{\mathbb{N}}
\newcommand{\R}{\mathbb{R}}
\newcommand{\Q}{\mathbb{Q}}
\newcommand{\Z}{\mathbb{Z}}
\newcommand{\abs}[1]{\left\vert #1 \right\vert}
\newcommand{\norm}[1]{\left\Vert #1 \right\Vert}
\DeclareMathOperator*{\argmax}{arg\,max}
\DeclareMathOperator*{\argmin}{arg\,min}
